﻿using FluentValidation;
using GAFNETLibrary.Enums;
using System.Collections.Generic;

namespace GAFNETBlazor.Models.Validations
{
    public class FaturamentoModelValidator : AbstractValidator<FaturamentoModel>
    {
        readonly int tamObs = 5000;
        readonly List<TipoFaturamento> tiposComFormulario = new() { TipoFaturamento.PedidoDeVenda, TipoFaturamento.OrdemDeServico, TipoFaturamento.Orcamento, TipoFaturamento.NotaFiscal };

        public FaturamentoModelValidator()
        {
            RuleFor(p => p.Pessoa.Id)
                .GreaterThan(0)
                .WithMessage("A pessoa deve ser informada");
            
            RuleFor(p => p.Historico.Id)
                .GreaterThan(0)
                .WithMessage("O histórico deve ser informado");
            
            RuleFor(p => p.Formulario.Id)
                .GreaterThan(0)
                .When(p => tiposComFormulario.Contains((TipoFaturamento)p.Tipo))
                .WithMessage("O formulário deve ser informado");
            
            RuleFor(p => p.Numero)
                .GreaterThan(0)
                .When(p => !tiposComFormulario.Contains((TipoFaturamento)p.Tipo))
                .WithMessage("O número do documento deve ser informado");
            
            RuleFor(p => p.Serie)
                .NotEmpty()
                .When(p => p.Tipo != (int)TipoFaturamento.PedidoDeCompra)
                .WithMessage("A série deve ser informada");
            
            RuleFor(p => p.Itens.Count)
                .GreaterThan(0)
                .WithMessage("Deve ser inserido ao menos um item");
            
            RuleFor(p => p.Observacoes)
                .MaximumLength(tamObs)
                .WithMessage($"A observação deve conter no máximo {tamObs} caracteres");
            
            RuleFor(p => p.CondicaoDePagamento.Id)
                .GreaterThan(0)
                .When(p => p.Historico.GeraFinanceiro && p.Historico.FinNFe != (int)FinNFeHistorico.Devolucao && p.Historico.TipoNF != (int)TipoNFHistorico.Retorno)
                .WithMessage("Este histórico exige uma condição de pagamento");
            
            RuleFor(p => p.CondicaoDePagamento.Id)
                .Equal(0)
                .When(p => !p.Historico.GeraFinanceiro)
                .WithMessage("Este histórico não permite condição de pagamento");
            
            //RuleFor(p => p.PlanoDeConta.Id)
            //    .GreaterThan(0)
            //    .When(p => p.Historico.GeraFinanceiro && p.Historico.FinNFe != (int)FinNFeHistorico.Devolucao && p.Historico.TipoNF != (int)TipoNFHistorico.Retorno)
            //    .WithMessage("Este histórico exige um plano de conta");
            
            //RuleFor(p => p.PlanoDeConta.Id)
            //    .Equal(0)
            //    .When(p => !p.Historico.GeraFinanceiro)
            //    .WithMessage("Este histórico não permite plano de conta");
            
            RuleFor(p => p.TipoDeOperacao.Id)
                .GreaterThan(0)
                .When(p => p.Historico.GeraFinanceiro && p.Historico.FinNFe != (int)FinNFeHistorico.Devolucao && p.Historico.TipoNF != (int)TipoNFHistorico.Retorno)
                .WithMessage("Este histórico exige um tipo de operação");
            
            RuleFor(p => p.TipoDeOperacao.Id)
                .Equal(0)
                .When(p => !p.Historico.GeraFinanceiro)
                .WithMessage("Este histórico não permite tipo de operação");
            
            RuleFor(p => p.CentroDeCusto.Id)
                .GreaterThan(0)
                .When(p => p.Historico.GeraFinanceiro && p.Historico.FinNFe != (int)FinNFeHistorico.Devolucao && p.Historico.TipoNF != (int)TipoNFHistorico.Retorno)
                .WithMessage("Este histórico exige um centro de custo");
            
            RuleFor(p => p.CentroDeCusto.Id)
                .Equal(0)
                .When(p => !p.Historico.GeraFinanceiro)
                .WithMessage("Este histórico não permite centro de custo");
        }
    }
}