﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services;

public class ConsultaPessoaAPIService : IConsultaPessoaAPIService
{
    public HttpClient HttpClient { get; set; }
    private readonly IHttpClientFactory _clientFactory;
    private readonly ICidadesService _cidadesService;
    private readonly IPaisesService _paisesService;


    private readonly IHttpClientComToken _httpClientComToken;
    public ConsultaPessoaAPIService(IHttpClientComToken httpClientComToken,
        IHttpClientFactory clientFactory,
        ICidadesService cidadesService,
        IPaisesService paisesService)
    {
        _httpClientComToken = httpClientComToken;
        _clientFactory = clientFactory;
        HttpClient = _clientFactory.CreateClient();
        _cidadesService = cidadesService;
        _paisesService = paisesService;
    }

    public async Task<RespostaAPIModel> GetByCNPJAsync(string cnpj)
    {
        cnpj = new String(cnpj.Where(Char.IsDigit).ToArray());

        RespostaAPIModel respostaAPI = await ConsultaPrimeiraAPI(cnpj);

        if (!respostaAPI.Sucesso)
        {
            respostaAPI = await ConsultaSegundaAPI(cnpj);
        }

        if (!respostaAPI.Sucesso)
        {
            return respostaAPI;
        }

        var retorno = JsonConvert.DeserializeObject<RetornoConsultaCNPJModel>(respostaAPI.ConteudoRetorno);

        var pessoa = new PessoaModel
        {
            Nome = retorno.Nome,
            NomeFantasia = retorno.Fantasia,
            CNPJ = retorno.CNPJ,
        };

        var retornoCidade = await _cidadesService.GetByNameAndUFAsync(URIsPessoas.GetCidadesAsync, retorno.Municipio, retorno.UF, StatusAtivo.Ativos);
        if (!retornoCidade.Sucesso)
        {
            return retornoCidade;
        }

        var cidade = (CidadeModel)retornoCidade.Objeto;

        var retornoPais = await _paisesService.GetPadraoAsync(URIsPessoas.GetPaisesAsync);
        if (!retornoPais.Sucesso)
        {
            return retornoPais;
        }

        var pais = (PaisModel)retornoPais.Objeto;

        pessoa.Enderecos.Add(new EnderecoPessoaModel
        {
            Logradouro = retorno.Logradouro,
            Numero = retorno.Numero,
            Bairro = retorno.Bairro,
            CEP = retorno.CEP,
            Complemento = retorno.Complemento,
            EhPrincipal = true,
            IdCidade = cidade.Id,
            Cidade = cidade,
            IdPais = pais.Id,
            Pais = pais,
            CNPJ = cnpj
        });

        pessoa.EnderecoPrincipal = pessoa.Enderecos.FirstOrDefault();

        if (!string.IsNullOrEmpty(retorno.Telefone))
        {
            var telefone = new string(retorno.Telefone.Where(Char.IsDigit).ToArray());
            pessoa.Telefones ??= new ObservableCollection<TelefonePessoaModel>();
            pessoa.Telefones.Add(new TelefonePessoaModel
            {
                Telefone = telefone,
            });
        }

        respostaAPI.Objeto = pessoa;

        return respostaAPI;
    }

    private async Task<RespostaAPIModel> ConsultaSegundaAPI(string cnpj)
    {
        var resposta = await HttpClient.GetAsync($"https://api-publica.speedio.com.br/buscarcnpj?cnpj={cnpj}");
        RespostaAPIModel respostaAPI = new(resposta);
        return respostaAPI;
    }

    private async Task<RespostaAPIModel> ConsultaPrimeiraAPI(string cnpj)
    {
        var resposta = await HttpClient.GetAsync($"http://www.receitaws.com.br/v1/cnpj/{cnpj}");

        RespostaAPIModel respostaAPI = new(resposta);

        string conteudo = "";

        if (!resposta.IsSuccessStatusCode && resposta.StatusCode != HttpStatusCode.TooManyRequests) // se o erro for por muitos requests, não sai do procedimento, mas segue para consultar na outra API
        {
            return respostaAPI;
        }

        if (respostaAPI.Sucesso)
        {
            conteudo = respostaAPI.ConteudoRetorno;
        }

        if (conteudo.Contains("ERROR"))
        {
            respostaAPI.Objeto = new PessoaModel();
            return respostaAPI;
        }
        return respostaAPI;
    }

}
