﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using System;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services;

public class ImportacaoTabelaIBPTService : IImportacaoTabelaIBPTService
{
    private readonly IHttpClientComToken _httpClientComToken;
    private readonly IWebHostEnvironment _env;
    public event Action<string> OnProcessarTabelaIBPT;

    public ImportacaoTabelaIBPTService(IHttpClientComToken httpClientComToken, IWebHostEnvironment env)
    {
        _httpClientComToken = httpClientComToken;
        _env = env;
    }
    public async Task<ResultadoModel> Importar()
    {
        var arquivos = Directory.GetFiles($@"{_env.WebRootPath}\files\TabelasIBPT", "*.csv");
        if (arquivos.Length == 0)
        {
            return new ResultadoModel { Sucesso = false, Retorno = "Não foram encontrados arquivos para processar" };
        }

        OnProcessarTabelaIBPT?.Invoke("Iniciando a importação...");

        foreach (var arq in arquivos)
        {
            await ProcessarTabela(arq);
        }

        OnProcessarTabelaIBPT?.Invoke("Importação encerrada");


        return new ResultadoModel { Sucesso = true, Retorno = "Procedimento de importação concluído. Verifique a lista de log." };
    }

    private async Task ProcessarTabela(string arq)
    {
        using var form = new MultipartFormDataContent();
        using var fs = File.OpenRead(arq);
        using var streamContent = new StreamContent(fs);
        using var fileContent = new ByteArrayContent(await streamContent.ReadAsByteArrayAsync());
        form.Add(fileContent, "arquivo", Path.GetFileName(arq));
        OnProcessarTabelaIBPT?.Invoke(Path.GetFileName(arq));

        try
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsTabelasIBPT.PostAsync, form, "multipart/form-data");
            if (!retorno.IsSuccessStatusCode)
            {
                OnProcessarTabelaIBPT?.Invoke($"Erro ao tentar importar o arquivo {Path.GetFileName(arq)}. Erro: {await retorno.Content.ReadAsStringAsync()}");
                return;
            }
            OnProcessarTabelaIBPT?.Invoke($"Arquivo {Path.GetFileName(arq)} importado com sucesso.");

            try
            {
                fs.Close();
                File.Delete(arq);
            }
            catch (Exception e)
            {
                OnProcessarTabelaIBPT?.Invoke($"Erro ao tentar excluir o arquivo {Path.GetFileName(arq)} importado. Erro: {(string)e.GetBaseException().Message}");
            }
        }
        catch (Exception e)
        {
            OnProcessarTabelaIBPT?.Invoke($"Erro ao tentar importar o arquivo {Path.GetFileName(arq)}. Erro: {(string)e.GetBaseException().Message}");
        }
    }

}
