﻿using BlazorDownloadFile;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class NFeService : INFeService
    {
        private readonly IBlazorDownloadFileService _blazorDownloadFileService;
        private readonly IHttpClientComToken _httpClientComToken;
        private readonly ITrocaArquivosService _trocaArquivoService;

        public NFeService(IBlazorDownloadFileService blazorDownloadFileService, IHttpClientComToken httpClientComToken, ITrocaArquivosService trocaArquivoService)
        {
            _blazorDownloadFileService = blazorDownloadFileService;
            _httpClientComToken = httpClientComToken;
            _trocaArquivoService = trocaArquivoService;
        }

        public event Action<string> OnEtapaEnvioNFe;
        public event Action<string> OnEtapaCancelamentoNFe;
        public event Action<string> OnEtapaInutilizacaoNFe;

        public async Task<ResultadoModel> CancelarNFeAsync(string chaveEletronica, string justificativa)
        {
            OnEtapaCancelamentoNFe?.Invoke("Cancelando a nota na receita...");
            var uri = URIsNFe.CancelamentoNFe;
            uri = new Uri(uri + $"?chaveeletronica={chaveEletronica}&&justificativa={justificativa}");
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, null);
            if (!retorno.IsSuccessStatusCode)
            {
                string erro = $"{await retorno.Content.ReadAsStringAsync()}";
                OnEtapaCancelamentoNFe?.Invoke(erro);
                return new ResultadoModel { Sucesso = false, Retorno = erro };
            }

            RetornoNFeModel ret;

            ret = JsonConvert.DeserializeObject<RetornoNFeModel>(await retorno.Content.ReadAsStringAsync());

            if (ret.CStat != 135)
            {
                string erro = $"Ocorreu um erro ao tentar cancelar a nota na receita. Erro: {ret.XMotivo}";
                OnEtapaCancelamentoNFe?.Invoke(erro);
            }
            else
            {
                OnEtapaCancelamentoNFe?.Invoke("Nota cancelada na receita com sucesso.");
            }

            return new ResultadoModel { Sucesso = true, Retorno = ret };
        }

        public async Task<ResultadoModel> InutilizarNFeAsync(ParamInutilizacaoNFeModel p)
        {
            OnEtapaInutilizacaoNFe?.Invoke("Inutilizando a nota na receita...");
            var uri = URIsNFe.InutilizacaoNFe;
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, p);
            if (!retorno.IsSuccessStatusCode)
            {
                string erro = $"Ocorreu um erro ao tentar enviar a Inutilização para a receita. Erro: {await retorno.Content.ReadAsStringAsync()}";
                OnEtapaInutilizacaoNFe?.Invoke(erro);
                return new ResultadoModel { Sucesso = false, Retorno = erro };
            }

            RetornoNFeModel ret;

            ret = JsonConvert.DeserializeObject<RetornoNFeModel>(await retorno.Content.ReadAsStringAsync());

            if (ret.CStat != 102)
            {
                string erro = $"Ocorreu um erro ao tentar inutilizar a nota na receita. Erro: status: {ret.CStat} descrição: {ret.XMotivo}";
                OnEtapaInutilizacaoNFe?.Invoke(erro);
            }
            else
            {
                OnEtapaInutilizacaoNFe?.Invoke("Nota inutilizada na receita com sucesso.");
            }

            return new ResultadoModel { Sucesso = true, Retorno = ret };
        }

        public Task<ResultadoModel> Consultar(string chave)
        {
            throw new NotImplementedException();
        }

        public async Task<ResultadoModel> Enviar(FaturamentoModel faturamento)
        {
            OnEtapaEnvioNFe?.Invoke("Gerando e enviando o XML para a receita...");
            var resultEnvio = await _httpClientComToken.Enviar(HttpMethod.Post, URIsNFe.EnvioNFe, faturamento);
            if (!resultEnvio.IsSuccessStatusCode)
            {
                string erro = await resultEnvio.Content.ReadAsStringAsync();
                OnEtapaEnvioNFe?.Invoke(erro);
                return new ResultadoModel { Sucesso = false, Retorno = erro };
            }

            RetornoNFeModel ret;

            ret = JsonConvert.DeserializeObject<RetornoNFeModel>(await resultEnvio.Content.ReadAsStringAsync());

            if (ret.CStat != 100)
            {
                string erro = $"Ocorreu um erro ao tentar aprovar a nota na receita. Erro: {ret.XMotivo}";
                OnEtapaEnvioNFe?.Invoke(erro);
                return new ResultadoModel { Sucesso = false, Retorno = erro };
            }

            OnEtapaEnvioNFe?.Invoke($"Nota aprovada na receita com a chave: { ret.ChDFe }");

            return new ResultadoModel { Sucesso = true, Retorno = ret };
        }

        public async Task<ResultadoModel> EnviarEMail(string chaveEletronica, TipoAnexoEMailNFe tipoAnexoEMail)
        {
            var uri = new Uri(URIsNFe.EnvioEMail + $"?chaveEletronica={chaveEletronica}&tipoAnexoEMail={(int)tipoAnexoEMail}");
            var resultEMail = await _httpClientComToken.GetResposta(uri);
            if (!resultEMail.IsSuccessStatusCode)
            {
                var conteudoRetorno = await resultEMail.Content.ReadAsStringAsync();
                return new ResultadoModel { Sucesso = false, Retorno = conteudoRetorno };
            }

            return new ResultadoModel { Sucesso = true, Retorno = "OK" };
        }

        public async Task<ResultadoModel> ImprimirDANFE(string chaveEletronica)
        {
            var uri = new Uri(URIsNFe.ImpressaoDANFE + $"?chaveEletronica={chaveEletronica}");
            string nomeArquivo = $"{chaveEletronica}-nfe.pdf";

            var resultDANFE = await _trocaArquivoService.Download(uri, nomeArquivo);
            if (!resultDANFE.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Ocorreu um erro ao tentar gerar e baixar a DANFE. Erro: {resultDANFE.Retorno}" };
            }

            return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
        }
    }
}
