﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services;

public class PessoasService : IPessoasService
{
    public HttpClient HttpClient { get; set; }
    private readonly IHttpClientFactory _clientFactory;
    private readonly IConsultaPessoaAPIService _consultaPessoaAPIService;
    private readonly ITimeZoneService _timeZoneService;

    const string expandGetById = "&$expand=historico($select=id,descricao),enderecoprincipal($expand=cidade,pais), enderecos($expand=cidade,pais),emails,telefones,produtos($expand=produto($select=id,codigo,nome))" +
                                 ",transportadoraspessoa($expand=transportadora),vendedorespessoa($expand=vendedor),centrodecusto($select=id,nome),tipodeoperacao($select=id,descricao)" +
                                 ",modeloboleto($select=id,nome),condicaodepagamento($select=id,descricao),planodeconta($select=id,descricao)";
    private readonly IHttpClientComToken _httpClientComToken;
    public PessoasService(IHttpClientComToken httpClientComToken, 
        IHttpClientFactory clientFactory,
        ITimeZoneService timeZoneService,
        IConsultaPessoaAPIService consultaPessoaAPIService)
    {
        _httpClientComToken = httpClientComToken;
        _clientFactory = clientFactory;
        HttpClient = _clientFactory.CreateClient();
        _timeZoneService = timeZoneService;
        _consultaPessoaAPIService = consultaPessoaAPIService;
    }

    public async Task<RespostaAPIModel> PostAsync<T>(T entity)
    {
        var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsPessoas.PostAsync, entity);
        return new RespostaAPIModel(retorno);
    }

    public async Task<RespostaAPIModel> PutAsync<T>(T entity)
    {
        var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsPessoas.PutAsync, entity);
        return new RespostaAPIModel(retorno);
    }

    public Task<HttpResponseMessage> GetClientesAsync(StatusAtivo statusAtividade = StatusAtivo.Todos)
    {
        return _httpClientComToken.GetResposta(URIsPessoas.GetClientesAsync);
    }

    public Task<HttpResponseMessage> GetFornecedoresAsync(StatusAtivo statusAtividade)
    {
        return _httpClientComToken.GetResposta(URIsPessoas.GetFornecedoresAsync);
    }

    public Task<HttpResponseMessage> GetTransportadorasAsync(StatusAtivo statusAtividade)
    {
        return _httpClientComToken.GetResposta(URIsPessoas.GetTransportadorasAsync);
    }

    public async Task<RespostaAPIModel> GetByCNPJAsync(string cnpj, StatusAtivo statusAtividade)
    {
        cnpj = new String(cnpj.Where(Char.IsDigit).ToArray());
        string select = $"?$Filter=cnpj eq \'{cnpj}\' &$Count=true";
        if (statusAtividade != StatusAtivo.Todos)
        {
            select = statusAtividade == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
        }

        select += expandGetById;

        var resposta = await _httpClientComToken.GetResposta(new Uri(URIsPessoas.GetAsync, select));

        RespostaAPIModel respostaAPI = new(resposta);

        if (!resposta.IsSuccessStatusCode) return respostaAPI;

        var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(respostaAPI.ConteudoRetorno);

        if (pessoas.Count > 0)
        {
            ConverterParaDataLocal(pessoas);
            respostaAPI.Objeto = pessoas.FirstOrDefault();
            return respostaAPI;
        }

        respostaAPI = await _consultaPessoaAPIService.GetByCNPJAsync(cnpj);


        if (pessoas.Count == 0)
        {
            respostaAPI.Sucesso = true;
            respostaAPI.MensagemErro = $"Não foi encontrada uma pessoa com o CNPJ {cnpj}.";
            respostaAPI.Status = HttpStatusCode.NotFound;
            return respostaAPI;
        }

        return respostaAPI;
    }

    

    public PessoaModel GetPessoaVazia()
    {
        var pessoa = new PessoaModel
        {
            Ativo                = true,
            Enderecos            = new ObservableCollection<EnderecoPessoaModel>(),
            EMails               = new ObservableCollection<EMailPessoaModel>(),
            Telefones            = new ObservableCollection<TelefonePessoaModel>(),
            VendedoresPessoa     = new ObservableCollection<VendedorPessoaModel>(),
            TransportadorasPessoa= new ObservableCollection<TransportadoraPessoaModel>(),
            Historico            = new HistoricoModel(),
            PlanoDeConta         = new PlanoDeContaModel(),
            CentroDeCusto        = new CentroDeCustoModel(),
            CondicaoDePagamento  = new CondicaoDePagamentoModel(),
            TipoDeOperacao       = new TipoDeOperacaoModel(),
            TipoRegimeTributacao = TipoRegimeTributacao.Simples,
            IndicadorIE          = IndicadorIE.Contribuinte,
            DtCadastro           = DateTime.Today
        };
        return pessoa;
    }

    public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
    {
        RespostaAPIModel respostaAPI = new();
        string select = uriBase + "?$Select=Id,Nome,CNPJ,Ativo,DtCadastro&$expand=enderecoprincipal($select=id;$expand=cidade($select=id,nome,uf))&$OrderBy=Nome&$Count=true";
        if (status != StatusAtivo.Todos)
        {
            select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
        }
        var resposta = await _httpClientComToken.GetResposta(new Uri(select));

        if (!resposta.IsSuccessStatusCode)
        {
            respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
            respostaAPI.Status = resposta.StatusCode;
            return respostaAPI;
        }

        respostaAPI.Sucesso = true;
        var dados = await resposta.Content.ReadAsStringAsync();
        var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(dados);

        ConverterParaDataLocal(pessoas);

        respostaAPI.Objeto = pessoas;
        respostaAPI.Status = pessoas.Count > 0 ? HttpStatusCode.OK : HttpStatusCode.NoContent;
        return respostaAPI;

    }

    private void ConverterParaDataLocal(List<PessoaModel> pessoas)
    {
        foreach (var pessoa in pessoas)
        {
            if (pessoa.DtCadastro.HasValue)
                pessoa.DtCadastro = _timeZoneService.GetLocalDateTime(pessoa.DtCadastro.Value);
            if (pessoa.DtNascimento.HasValue)
                pessoa.DtNascimento = _timeZoneService.GetLocalDateTime(pessoa.DtNascimento.Value);
        }
    }

    public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
    {
        string select = uriBase + $"?$Count=true&$Filter=id eq {id}";
        if (status != StatusAtivo.Todos)
        {
            select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
        }

        select += expandGetById;

        var resposta = await _httpClientComToken.GetResposta(new Uri(select));

        RespostaAPIModel respostaAPI = new(resposta);

        if (!resposta.IsSuccessStatusCode)
        {
            return respostaAPI;
        }

        var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(respostaAPI.ConteudoRetorno);
        if (pessoas.Count == 0)
        {
            respostaAPI.Sucesso = false;
            respostaAPI.MensagemErro = $"Não foi encontrada uma pessoa com o id {id}.";
            respostaAPI.Status = HttpStatusCode.NotFound;
            return respostaAPI;
        }

        ConverterParaDataLocal(pessoas);
        respostaAPI.Objeto = pessoas.FirstOrDefault();

        return respostaAPI;
    }

    public async Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
    {
        string select = uriBase + "?$Count=true";
        select = !string.IsNullOrEmpty(campos) ? select += $"&$select={campos}" : select;
        select = !string.IsNullOrEmpty(filtros) ? select += $"&$filter={filtros}" : select;
        select = !string.IsNullOrEmpty(order) ? select += $"&$orderby={order}" : select;
        select = !string.IsNullOrEmpty(top) && top != "0"? select += $"&$top={top}" : select;
        var resposta = await _httpClientComToken.GetResposta(new Uri(select));

        RespostaAPIModel respostaAPI = new(resposta);
        if (resposta.IsSuccessStatusCode)
        {
            var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = pessoas;
            respostaAPI.Status = pessoas.Count > 0 ? HttpStatusCode.OK : HttpStatusCode.NoContent;
        }

        return respostaAPI;
    }

    public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, string campos, string filtros)
    {
        RespostaAPIModel respostaAPI = new();

        string queryString = !string.IsNullOrEmpty(campos) ? uriBase + $"?$Select={campos}" : uriBase + "?";
        queryString = !string.IsNullOrEmpty(campos) ? queryString += $"&$Filter=id eq {id}" : queryString += $"$Filter=id eq {id}";
        queryString = !string.IsNullOrEmpty(filtros) ? queryString += $" and {filtros}" : queryString;
        queryString += "&$count=true";
        var resposta = await _httpClientComToken.GetResposta(new Uri(queryString));

        if (!resposta.IsSuccessStatusCode)
        {
            respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
            respostaAPI.Status = resposta.StatusCode;
            return respostaAPI;
        }
        
        var dados = await resposta.Content.ReadAsStringAsync();
        var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(dados);
        if (pessoas.Count == 0)
        {
            respostaAPI.MensagemErro = $"Não foi encontrada uma pessoa com o id {id}.";
            respostaAPI.Status = HttpStatusCode.NotFound;
            return respostaAPI;
        }
        ConverterParaDataLocal(pessoas);
        respostaAPI.Status = resposta.StatusCode;
        respostaAPI.Sucesso = true;
        respostaAPI.Objeto = pessoas.FirstOrDefault();

        return respostaAPI;
    }

    public async Task<RespostaAPIModel> GetAsync(PesquisaPessoaModel parametros)
    {
        string select = URIsPessoas.GetAsync + "?$Select=Id,Nome,CNPJ,Ativo,DtCadastro&$expand=enderecoprincipal($select=id;$expand=cidade($select=id,nome,uf)),telefones";

        string filter = $"&$filter=id ge 0";
        if (parametros.Endereco.IdCidade > 0) filter += $" and enderecos/any(e: e/idcidade eq {parametros.Endereco.IdCidade})";
        if (parametros.Historico.Id > 0) filter += $" and IdHistorico eq {parametros.Historico.Id}";

        string strFiltroPessoa = "";
        if (!string.IsNullOrEmpty(parametros.Nome))
        {
            strFiltroPessoa = (TipoPesquisaString)parametros.TipoPesquisaNome switch
            {
                TipoPesquisaString.Exato => $" and nome eq '{parametros.Nome}')",
                TipoPesquisaString.ComecaCom => $" and startswith(nome, '{parametros.Nome}')",
                TipoPesquisaString.TerminaCom => $" and endswith(nome, '{parametros.Nome}')",
                TipoPesquisaString.Contem => $" and substringof(nome, '{parametros.Nome}')",
                _ => $" and nome eq '{parametros.Nome}')"
            };
        }

        filter += strFiltroPessoa;
        filter += parametros.EhCliente ? " and ehcliente eq true" : "";
        filter += parametros.EhFornecedor ? " and ehfornecedor eq true" : "";
        filter += parametros.EhTransportadora ? " and ehtransportadora eq true" : "";
        filter += parametros.EhVendedor ? " and ehvendedor eq true" : "";
        filter += parametros.Ativo ? " and ativo eq true" : "";
        filter += parametros.MEI ? " and mei eq true" : "";

        select += filter;

        if (parametros.QtdRegistros > 0) select += $"&$top={parametros.QtdRegistros}";

        select += string.IsNullOrEmpty(parametros.Ordem) ? "&$OrderBy=Nome" : $"&$OrderBy={parametros.Ordem}";

        var resposta = await _httpClientComToken.GetResposta(new Uri(select));
        var respostaAPI = new RespostaAPIModel(resposta);

        if (!resposta.IsSuccessStatusCode)
        {
            return respostaAPI;
        }

        var pessoas = JsonConvert.DeserializeObject<List<PessoaModel>>(respostaAPI.ConteudoRetorno);
        respostaAPI.Objeto = pessoas;
        respostaAPI.Status = pessoas.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
        return respostaAPI;
    }

}
