﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class ProdutosService : IProdutosService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public ProdutosService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase?.ToString() + "?$Select=id,codigo,nome,tipo,vlrvenda,datacadastro,idclassificacaofiscal,idunidade,ativo&$Expand=subgrupo($select=id,nome;$expand=grupo($select=id,nome))&$OrderBy=nome&$Count=true";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            string conteudo = await resposta.Content.ReadAsStringAsync();

            var retornoAPI = new RespostaAPIModel(resposta);
            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            List<ProdutoModel> produtos = new ();
            
            produtos = JsonConvert.DeserializeObject<List<ProdutoModel>>(conteudo);

            if (!produtos.Any())
            {
                retornoAPI.Status = HttpStatusCode.NoContent;
                retornoAPI.MensagemErro = "Não foram encontrados produtos.";
                return retornoAPI;
            }
            
            retornoAPI.Objeto = produtos;
            retornoAPI.Status = HttpStatusCode.OK;
            retornoAPI.Sucesso = true;

            return retornoAPI;
        }

        public Task<RespostaAPIModel> GetByCodigoAsync(Uri uriBase, string codigo, string campos, StatusAtivo status)
        {
            return ProcedimentoGetByCodigoAsync(uriBase, codigo, campos, status);
        }

        public Task<RespostaAPIModel> GetByCodigoAsync(Uri uriBase, string codigo, StatusAtivo status)
        {
            return ProcedimentoGetByCodigoAsync(uriBase, codigo, "", status);
        }

        private async Task<RespostaAPIModel> ProcedimentoGetByCodigoAsync(Uri uriBase, string codigo, string campos, StatusAtivo status)
        {
            string filter = $"& $Filter = codigo eq \'{codigo}\'";
            string select = uriBase + "?$Count=true ";
            campos = string.IsNullOrEmpty(campos) ? campos : $"& $select={campos} ";
            select += string.IsNullOrEmpty(campos) ? "& $expand=aliquota($select=id,descricao), unidade($select=id,codigo), classificacaofiscal($select=id,numero), subgrupo($expand=grupo)" : campos;
            select += filter;
            RespostaAPIModel respostaAPI = new ();

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }
            
            var dados = await resposta.Content.ReadAsStringAsync();
            var produtos = JsonConvert.DeserializeObject<List<ProdutoModel>>(dados);
            if (!produtos.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um item com o código {codigo}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }
            respostaAPI.Objeto = produtos.FirstOrDefault();
            respostaAPI.Sucesso = true;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            RespostaAPIModel respostaAPI = new ();
            string select = uriBase + $"?$Count=true & $expand=aliquota($select=id,descricao), unidade($select=id,codigo), classificacaofiscal($select=id,numero), subgrupo($expand=grupo)  & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }
            
            var dados = await resposta.Content.ReadAsStringAsync();
            var produtos = JsonConvert.DeserializeObject<List<ProdutoModel>>(dados);
            if (!produtos.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um item com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }
            respostaAPI.Objeto = produtos.FirstOrDefault();
            respostaAPI.Sucesso = true;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            string select = uriBase + "?$Count=true";
            select = !string.IsNullOrEmpty(campos) ? select += $"&$select={campos}" : select;
            select = !string.IsNullOrEmpty(filtros) ? select += $"&$filter={filtros}" : select;
            select = !string.IsNullOrEmpty(order) ? select += $"&$orderby={order}" : select;
            select = !string.IsNullOrEmpty(top) ? select += $"&$top={top}" : select;
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new(resposta);
            if (resposta.IsSuccessStatusCode)
            {
                var produtos = JsonConvert.DeserializeObject<List<ProdutoModel>>(respostaAPI.ConteudoRetorno);
                respostaAPI.Objeto = produtos;
                respostaAPI.Status = produtos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            }

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsProdutos.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsProdutos.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
