﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroFormulario
{
    public class CadastroFormularioVM : CadastroBaseVM, ICadastroFormularioVM
    {
        private readonly IFormulariosService _formulariosService;
        private readonly ILoginStore _loginStore;

        private bool _dlgFormulariosVisible;
        private FormularioModel _formulario;
        private List<ListaModel> _tiposFormulario;

        public FormularioModel Formulario
        {
            get { return _formulario; }
            set
            {
                _formulario = value;
                AoMudarPropriedade(nameof(Formulario));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimFormulario = _loginStore.Claims.FirstOrDefault(c => c.Type == "Formulario");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimFormulario != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgFormulariosVisible
        {
            get { return _dlgFormulariosVisible; }
            set
            {
                _dlgFormulariosVisible = value;
                AoMudarPropriedade(nameof(DlgFormulariosVisible));
            }
        }

        public List<ListaModel> TiposFormulario
        {
            get { return _tiposFormulario; }
            set
            {
                _tiposFormulario = value;
                AoMudarPropriedade(nameof(TiposFormulario));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox NomeTextBox { get; set; }
        public FormularioModel FormularioSelecionado { get; set; } = new FormularioModel();
        public List<FormularioModel> Formularios { get; set; } = new List<FormularioModel>();

        public CadastroFormularioVM(IFormulariosService formulariosService, ITiposFormularioService tiposFormularioService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _formulariosService = formulariosService;
            Formulario = new FormularioModel { Ativo = true };
            MensagemResultadoEnvio = " ";
            TiposFormulario = tiposFormularioService.Itens();
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroFormulario.Enviar(this, _formulariosService);
        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new FormularioModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Formulario);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgFormularios(object e)
        {
            DlgFormulariosVisible = false;
        }

        public async void AoClicarOkBtnDlgFormularios(MouseEventArgs e)
        {
            if (FormularioSelecionado == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgFormulariosVisible = false;
            Formulario = await FormularioByIdCadastroFormulario.Get(this, _formulariosService, FormularioSelecionado.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarFormulario(RowSelectEventArgs<FormularioModel> e)
        {
            FormularioSelecionado = e.Data;
        }

        public async Task<List<FormularioModel>> GetFormularios()
        {
            var retorno = await _formulariosService.GetAsync(URIsFormularios.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<FormularioModel>();
            }
            return (List<FormularioModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Formularios = await GetFormularios();
            Localizando = false;
            DlgFormulariosVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Formulario.Id == 0) return;
            Formulario = await FormularioByIdCadastroFormulario.Get(this, _formulariosService, Formulario.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroFormulario.Limpar(this);
        }
    }
}
