﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPlanoDeConta;

public partial class CadastroPlanoDeContaVM : CadastroBaseVM, ICadastroPlanoDeContaVM
{
    private readonly IPlanosDeContaService _planosDeContaService;
    private readonly ILoginStore _loginStore;

    [ObservableProperty]
    private List<ListaModel> tiposPlanoDeConta;
    partial void OnTiposPlanoDeContaChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposPlanoDeConta));

    [ObservableProperty]
    private List<ListaModel> tiposDebitoCreditoConta;
    partial void OnTiposDebitoCreditoContaChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposDebitoCreditoConta));

    [ObservableProperty]
    private List<ListaModel> tiposDespesaPlanoDeConta;
    partial void OnTiposDespesaPlanoDeContaChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposDespesaPlanoDeConta));

    [ObservableProperty]
    private PlanoDeContaModel planoDeConta;
    partial void OnPlanoDeContaChanged(PlanoDeContaModel value) => AoMudarPropriedade(nameof(PlanoDeConta));

    [ObservableProperty]
    private bool contasContrapartidaBtnEnabled;
    partial void OnContasContrapartidaBtnEnabledChanged(bool value) => AoMudarPropriedade(nameof(ContasContrapartidaBtnEnabled));

    [ObservableProperty]
    private bool pesquisandoContasContrapartida;
    partial void OnPesquisandoContasContrapartidaChanged(bool value) => AoMudarPropriedade(nameof(PesquisandoContasContrapartida));

    public bool PodeAcessar
    {
        get
        {
            var claimPlanoDeConta = _loginStore.Claims.FirstOrDefault(c => c.Type == "Plano de Conta");
            return _loginStore.EhAdmin || _loginStore.EhRepresentante || claimPlanoDeConta != null;
        }
    }

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    [ObservableProperty]
    private bool dlgPlanosDeContaVisible;
    partial void OnDlgPlanosDeContaVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPlanosDeContaVisible));
    [ObservableProperty]
    private bool dlgContasContrapartidaVisible;
    partial void OnDlgContasContrapartidaVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgContasContrapartidaVisible));

    public SfTextBox DescricaoTextBox { get; set; }
    public SfNumericTextBox<long?> IdContaContrapartidaInput { get; set; }
    public PlanoDeContaModel PlanoDeContaSelecionado { get; set; } = new PlanoDeContaModel();
    public PlanoDeContaModel ContaContrapartidaSelecionada { get; set; } = new PlanoDeContaModel();
    public List<PlanoDeContaModel> PlanosDeConta { get; set; } = new List<PlanoDeContaModel>();

    public CadastroPlanoDeContaVM(IPlanosDeContaService planosDeContaService, 
        ILoginStore loginStore, 
        ITiposPlanoDeContaService tiposPlanoDeContaService, 
        ITiposDebitoCreditoContaService tiposDebitoCreditoContaService, 
        ITiposDespesaPlanoDeContaService tiposDespesaPlanoDeContaService)
    {
        _loginStore = loginStore;
        _planosDeContaService = planosDeContaService;
        PlanoDeConta = new PlanoDeContaModel { Ativo = true };
        PlanoDeConta.ContaContrapartida = new PlanoDeContaModel ();
        MensagemResultadoEnvio = " ";
        TiposPlanoDeConta = tiposPlanoDeContaService.Itens();
        TiposDebitoCreditoConta = tiposDebitoCreditoContaService.Itens();
        TiposDespesaPlanoDeConta = tiposDespesaPlanoDeContaService.Itens();
        ContasContrapartidaBtnEnabled = true;
    }

    public async void Enviar()
    {
        if (!DadosValidos())
        {
            return;
        }

        await EnvioDadosCadastroPlanoDeConta.Enviar(this, _planosDeContaService);
    }

    public async void AoClicarNovoRegistroBtn(MouseEventArgs e)
    {
        LimparTela();
        await Task.Delay(100);
        await DescricaoTextBox.FocusAsync();
    }

    public bool DadosValidos()
    {
        var validator = new PlanoDeContaModelValidator();
        ErrosValidacaoFormulario.Clear();
        List<string> erros = new();

        var results = validator.Validate(PlanoDeConta);
        if (!results.IsValid)
        {
            foreach (var erro in results.Errors)
            {
                erros.Add(erro.ErrorMessage);
            }
            ErrosValidacaoFormulario.AddRange(erros);
            return false;
        }
        return true;
    }

    public void AoFecharDlgPlanosDeConta(object e)
    {
        DlgPlanosDeContaVisible = false;
    }

    public void AoSelecionarPlanoDeConta(RowSelectEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.Data;
    }

    public async Task<List<PlanoDeContaModel>> GetPlanosDeConta()
    {
        var retorno = await _planosDeContaService.GetAsync(URIsPlanosDeConta.GetAsync, StatusAtivo.Todos);

        if (!string.IsNullOrEmpty(retorno.MensagemErro))
        {
            Mensagem = retorno.MensagemErro;
            return new List<PlanoDeContaModel>();
        }
        return (List<PlanoDeContaModel>)retorno.Objeto;
    }

    public async void AoClicarLocalizarBtn(MouseEventArgs e)
    {
        Localizando = true;
        PlanosDeConta = await GetPlanosDeConta();
        Localizando = false;
        DlgPlanosDeContaVisible = true;
    }

    public async void IdAoSairAsync()
    {
        ErrosValidacaoFormulario.Clear();
        if (PlanoDeConta.Id == 0) return;
        PlanoDeConta = await PlanoDeContaByIdCadastroPlanoDeContas.Get(this, _planosDeContaService, PlanoDeConta.Id, StatusAtivo.Todos);
    }

    public void LimparTela()
    {
        LimpaTelaCadastroPlanoDeConta.Limpar(this);
    }

    public void AoFecharDlg(object e)
    {
        throw new System.NotImplementedException();
    }

    public async void IdContaContrapartidaAoSairAsync()
    {
        PesquisandoContasContrapartida = true;
        if (!PlanoDeConta.IdContaContrapartida.HasValue || PlanoDeConta.IdContaContrapartida.Value == 0)
        {
            return;
        }
        PlanoDeConta.ContaContrapartida = await ContaContrapartidaByIdCadastroPlanoDeContas.Get(this, _planosDeContaService, PlanoDeConta.IdContaContrapartida.Value, StatusAtivo.Ativos);
        PesquisandoContasContrapartida = false;
    }

    public async void AoClicarContasContrapartidaBtnAsync(MouseEventArgs e)
    {
        PesquisandoContasContrapartida = true;
        PlanosDeConta = await GetPlanosDeConta();
        PesquisandoContasContrapartida = false;
        DlgContasContrapartidaVisible = true;
    }

    public void AoFecharDlgContasContrapartida(object e)
    {
        DlgContasContrapartidaVisible = false;
    }

    public async void AoClicarOkBtnDlgContasContrapartida(MouseEventArgs e)
    {
        if (ContaContrapartidaSelecionada == null)
        {
            return;
        }
        DlgContasContrapartidaVisible = false;
        PlanoDeConta.ContaContrapartida = await PlanoDeContaByIdCadastroPlanoDeContas.Get(this, _planosDeContaService, ContaContrapartidaSelecionada.Id, StatusAtivo.Ativos);
    }

    public async void AoClicarSelecionarBtnGridPlanosDeConta(CommandClickEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.RowData;
        await ExibirPlanoDeContaSelecionado();
    }

    public async void AoClicarSelecionarBtnGridContasContrapartida(CommandClickEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.RowData;
        await ExibirContaContrapartidaSelecionada();
    }

    public async void AoClicarOkBtnDlgPlanosDeConta(MouseEventArgs e)
    {
        if (PlanoDeContaSelecionado == null)
        {
            return;
        }
        await ExibirPlanoDeContaSelecionado();
    }

    private async Task ExibirPlanoDeContaSelecionado()
    {
        ErrosValidacaoFormulario.Clear();
        DlgPlanosDeContaVisible = false;
        PlanoDeConta = await PlanoDeContaByIdCadastroPlanoDeContas.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id, StatusAtivo.Todos);
    }

    private async Task ExibirContaContrapartidaSelecionada()
    {
        DlgContasContrapartidaVisible = false;
        PlanoDeConta.ContaContrapartida = await ContaContrapartidaByIdCadastroPlanoDeContas.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id, StatusAtivo.Todos);
    }

    public async Task AoPressionarTeclaPlanosDeContaGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && PlanoDeContaSelecionado?.Id > 0)
        {
            await ExibirPlanoDeContaSelecionado();
            await DescricaoTextBox.FocusAsync();
        }
    }

    public async Task AoPressionarTeclaContasContrapartidaGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && PlanoDeContaSelecionado?.Id > 0)
        {
            await ExibirContaContrapartidaSelecionada();
        }
    }

}
