﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroUnidadeProduto
{
    public class CadastroUnidadeProdutoVM : CadastroBaseVM, ICadastroUnidadeProdutoVM
    {
        private readonly IUnidadesProdutoService _unidadesService;
        private readonly ILoginStore _loginStore;

        private bool _dlgUnidadesVisible;
        private UnidadeProdutoModel _unidade;

        public UnidadeProdutoModel Unidade
        {
            get { return _unidade; }
            set
            {
                _unidade = value;
                AoMudarPropriedade(nameof(Unidade));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimUnidade = _loginStore.Claims.FirstOrDefault(c => c.Type == "UnidadeProduto");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimUnidade != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgUnidadesVisible
        {
            get { return _dlgUnidadesVisible; }
            set
            {
                _dlgUnidadesVisible = value;
                AoMudarPropriedade(nameof(DlgUnidadesVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox CodigoTextBox { get; set; }
        public SfTextBox DescricaoTextBox { get; set; }
        public UnidadeProdutoModel UnidadeSelecionada { get; set; } = new UnidadeProdutoModel();
        public List<UnidadeProdutoModel> Unidades { get; set; } = new List<UnidadeProdutoModel>();

        public CadastroUnidadeProdutoVM(IUnidadesProdutoService unidadesService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _unidadesService = unidadesService;
            Unidade = new UnidadeProdutoModel();
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroUnidadeProduto.Enviar(this, _unidadesService);
        }

        public void AoCriarCodigoTextBox(object e)
        {
            CodigoTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new UnidadeProdutoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Unidade);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgUnidades(object e)
        {
            DlgUnidadesVisible = false;
        }

        public async void AoClicarOkBtnDlgUnidades(MouseEventArgs e)
        {
            await ExibirUnidadeSelecionada();
        }

        private async Task ExibirUnidadeSelecionada()
        {
            if (UnidadeSelecionada == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgUnidadesVisible = false;
            Unidade = await UnidadeByIdCadastroUnidadeProduto.Get(this, _unidadesService, UnidadeSelecionada.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarUnidade(RowSelectEventArgs<UnidadeProdutoModel> e)
        {
            UnidadeSelecionada = e.Data;
        }

        public async Task<List<UnidadeProdutoModel>> GetUnidades()
        {
            var retorno = await _unidadesService.GetAsync(URIsUnidadesProduto.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<UnidadeProdutoModel>();
            }
            return (List<UnidadeProdutoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Unidades = await GetUnidades();
            Localizando = false;
            DlgUnidadesVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Unidade.Id == 0) return;
            Unidade = await UnidadeByIdCadastroUnidadeProduto.Get(this, _unidadesService, Unidade.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroUnidadeProduto.Limpar(this);
        }

        public async void AoClicarSelecionarBtnGridUnidades(CommandClickEventArgs<UnidadeProdutoModel> e)
        {
            UnidadeSelecionada = e.RowData;
            await ExibirUnidadeSelecionada();
            await DescricaoTextBox.FocusAsync();
        }
    }
}
