﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Faturamento;

public static class CancelamentoDocumento
{
    public static async Task Cancelar(BaseVM viewModel, FaturamentoModel faturamento, INFeService nFeService, IEnvioFaturamentoService envioFaturamentoService)
    {
        try
        {
            await viewModel.SpinnerPrincipal.ShowAsync();
            await ExecutarCodigo(viewModel, faturamento, nFeService, envioFaturamentoService);
        }
        finally
        {
            await viewModel.SpinnerPrincipal.HideAsync();
        }

    }

    private static async Task ExecutarCodigo(BaseVM viewModel, FaturamentoModel faturamento, INFeService nFeService, IEnvioFaturamentoService envioFaturamentoService)
    {
        if (faturamento.DtCancelamento.HasValue)
        {
            await viewModel.ExibirMensagemToast("Documento cancelado. Não é possível processar.", TipoToast.Aviso);
            return;
        }

        ResetarLog(viewModel);
        ExibirDlgLog(viewModel);

        ResultadoModel deveCancelarNoBanco = new ResultadoModel { Sucesso = true };
        //Primeiro cancela a nota na receita e somente se tiver sucesso, realiza o cancelamento da nota no banco
        if (!string.IsNullOrEmpty(faturamento.ChaveEletronica))
        {
            deveCancelarNoBanco = await CancelarNFe(faturamento, nFeService);
        }

        if (!deveCancelarNoBanco.Sucesso)
        {
            string erro = (string)deveCancelarNoBanco.Retorno;
            await viewModel.ExibirMensagemToast((string)deveCancelarNoBanco.Retorno, TipoToast.Erro);
            return;
        }

        SetarLog(viewModel, $"\n\rCancelando o documento no banco...");

        var resultado = await envioFaturamentoService.CancelarAsync(faturamento.Id, (TipoFaturamento)faturamento.Tipo);

        if (!resultado.Sucesso)
        {
            SetarLog(viewModel, $"\n\rErro: {(string)resultado.Retorno}");
            return;
        }

        var retornoFat = (FaturamentoModel)resultado.Retorno;
        faturamento.DtCancelamento = retornoFat.DtCancelamento;

        SetarLog(viewModel, $"\n\rdocumento cancelado com sucesso no banco");

        if (viewModel is FaturamentoVM fatVM)
        {
            await fatVM.PreencherFormularioPadrao();
        }

        await viewModel.ExibirMensagemToast($"Cancelamento realizado com sucesso.", TipoToast.Sucesso);
    }

    private static async Task<ResultadoModel> CancelarNFe(FaturamentoModel faturamento, INFeService nFeService)
    {
        var resultCancelNFe = await nFeService.CancelarNFeAsync(faturamento.ChaveEletronica, faturamento.MotivoCancelamento);

        faturamento.MotivoCancelamento = "";

        if (!resultCancelNFe.Sucesso) return resultCancelNFe;

        var ret = (RetornoNFeModel)resultCancelNFe.Retorno;
        bool deveCancelarNoBanco = ((ret.CStat == 135 || ret.CStat == 573) && !faturamento.DtCancelamento.HasValue);
        return new ResultadoModel { Sucesso = deveCancelarNoBanco };
    }

    private static void ExibirDlgLog(BaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).DlgLogEnvioCancelamentoVisible = true;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).DlgLogEnvioCancelamentoVisible = true;
                break;
            default:
                break;
        }
    }

    private static void SetarLog(BaseVM viewModel, string mensagem)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).LogEnvioCancelamento += mensagem;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).LogEnvioCancelamento += mensagem;
                break;
            default:
                break;
        }

    }

    private static void ResetarLog(BaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).LogEnvioCancelamento = "";
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).LogEnvioCancelamento = "";
                break;
            default:
                break;
        }

    }
}
