﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Faturamento;

public static class InutilizacaoDocumento
{
    public static async Task Inutilizar(BaseVM viewModel, FaturamentoModel faturamento, INFeService nFeService, IEnvioFaturamentoService envioFaturamentoService)
    {
        if (!faturamento.Formulario.NFEletronica)
        {
            await viewModel.ExibirMensagemToast("Esta nota não é eletrônica. Não pode ser inutilizada.", TipoToast.Aviso);
            return;
        }
        
        try
        {
            await viewModel.SpinnerPrincipal.ShowAsync();
            await ExecutarCodigo(viewModel, faturamento, nFeService, envioFaturamentoService);
        }
        finally
        {
            await viewModel.SpinnerPrincipal.HideAsync();
        }
    }

    private static async Task ExecutarCodigo(BaseVM viewModel, FaturamentoModel faturamento, INFeService nFeService, IEnvioFaturamentoService envioFaturamentoService)
    {
        ResetarLog(viewModel);
        ExibirDlgLog(viewModel);
        //Primeiro inutiliza a nota na receita e somente se tiver sucesso, realiza o Inutilizacao da nota no banco
        
        ResultadoModel deveInutilizarNoBanco = new ResultadoModel { Sucesso = true };
        deveInutilizarNoBanco = await InutilizarNFe(faturamento, nFeService);


        if (!deveInutilizarNoBanco.Sucesso) 
        {
            string erro = (string)deveInutilizarNoBanco.Retorno;
            if (string.IsNullOrEmpty(erro)) return; //se não voltou um erro propriamente, não mostra a mensagem
            await viewModel.ExibirMensagemToast(erro, TipoToast.Erro);
            return;
        }

        SetarLog(viewModel, $"\n\rInutilizando o documento no banco...");

        var resultado = await envioFaturamentoService.InutilizarAsync(faturamento.Id);

        if (!resultado.Sucesso)
        {
            SetarLog(viewModel, $"\n\rErro: {(string)resultado.Retorno}");
            return;
        }

        var retornoFat = (FaturamentoModel)resultado.Retorno;
        faturamento.DtInutilizacao = retornoFat.DtInutilizacao;

        SetarLog(viewModel, $"\n\rdocumento inutilizado com sucesso no banco");

        if (viewModel is FaturamentoVM fatVM)
        {
            await fatVM.PreencherFormularioPadrao();
        }
        viewModel.AoMudarPropriedade(nameof(Faturamento));
        
        await viewModel.ExibirMensagemToast($"Inutilizacao realizada com sucesso.", TipoToast.Sucesso);
    }

    private static void ResetarLog(BaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).LogEnvioInutilizacao = "";
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).LogEnvioInutilizacao = "";
                break;
            default:
                break;
        }

    }

    private static void SetarLog(BaseVM viewModel, string mensagem)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).LogEnvioInutilizacao += mensagem;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).LogEnvioInutilizacao += mensagem;
                break;
            default:
                break;
        }

    }

    private static void ExibirDlgLog(BaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is FaturamentoVM:
                ((FaturamentoVM)fatVM).DlgLogEnvioInutilizacaoVisible = true;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)pesqVM).DlgLogEnvioInutilizacaoVisible = true;
                break;
            default:
                break;
        }

    }

    private static async Task<ResultadoModel> InutilizarNFe(FaturamentoModel faturamento, INFeService nFeService)
    {
        ParamInutilizacaoNFeModel p = new()
        {
            Ano = faturamento.DtHrLancamento.Year,
            Justificativa = faturamento.MotivoInutilizacao,
            Modelo = 55,
            NumeroInicial = faturamento.Numero,
            NumeroFinal = faturamento.Numero,
            Serie = faturamento.Serie
        };

        var resultInutNFe = await nFeService.InutilizarNFeAsync(p);

        faturamento.MotivoInutilizacao = "";

        if (!resultInutNFe.Sucesso) return resultInutNFe;

        var ret = (RetornoNFeModel)resultInutNFe.Retorno;

        bool deveInutilizarNoBanco = ((ret.CStat == 102 || ret.CStat == 563) && !faturamento.DtInutilizacao.HasValue);
        return new ResultadoModel { Sucesso = deveInutilizarNoBanco };
    }
}
