﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.Utilitarios;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.Extensions.Configuration;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Parametrizacao;

public class ParametrizacaoVM : CadastroBaseVM, IParametrizacaoVM
{
    private readonly IParametrizacoesService _parametrizacoesService;
    private readonly ILoginStore _loginStore;
    private readonly IUploadCertificadoDigitalService _uploadCertificadoDigitalService;
    private readonly IUploadLogoDANFENFeService _uploadLogoDANFENFeService;
    private readonly IConfiguration _configuration;

    private ParametrizacaoModel _parametrizacao;

    public ParametrizacaoModel Parametrizacao
    {
        get { return _parametrizacao; }
        set
        {
            _parametrizacao = value;
            AoMudarPropriedade(nameof(Parametrizacao));
        }
    }

    public bool PodeAcessar
    {
        get
        {
            var claimParametrizacao = _loginStore.Claims.FirstOrDefault(c => c.Type == "Parametrizacao");
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimParametrizacao != null);
        }
    }

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public SfDialog Dlg { get; set; }
    public IEnumerable<ListaModel> TiposEmissaoNFe { get; set; }
    public IEnumerable<ListaModel> TiposAmbienteNFe { get; set; }

    public ParametrizacaoVM(IParametrizacoesService parametrizacoesService, ILoginStore loginStore,
        IUploadCertificadoDigitalService uploadCertificadoDigitalService, IUploadLogoDANFENFeService uploadLogoDANFENFeService, IConfiguration configuration)
    {
        _loginStore = loginStore;
        _parametrizacoesService = parametrizacoesService;
        _uploadCertificadoDigitalService = uploadCertificadoDigitalService;
        _uploadLogoDANFENFeService = uploadLogoDANFENFeService;
        _configuration = configuration;
        Parametrizacao = new ParametrizacaoModel();
        MensagemResultadoEnvio = " ";
    }

    public async void Enviar()
    {
        if (!DadosValidos())
        {
            return;
        }

        string chave = _configuration.GetSection("EncryptKey").Value;
        Parametrizacao.SenhaCertificadoNFe = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.EncriptaString(chave, Parametrizacao.SenhaCertificadoNFe) : null;
        Parametrizacao.SenhaEMail = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.EncriptaString(chave, Parametrizacao.SenhaEMail) : null;

        var resultEnvio = await EnvioDadosParametrizacao.Enviar(this, _parametrizacoesService);
        await ExibirMensagemToast((string)resultEnvio.Retorno, resultEnvio.Sucesso ? TipoToast.Sucesso : TipoToast.Erro);

        //volta a decriptar para os campos exibirem normalmente
        Parametrizacao.SenhaCertificadoNFe = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.DecriptaString(chave, Parametrizacao.SenhaCertificadoNFe) : null;
        Parametrizacao.SenhaEMail = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.DecriptaString(chave, Parametrizacao.SenhaEMail) : null;
        AoMudarPropriedade(nameof(Parametrizacao));
    }

    public bool DadosValidos()
    {
        var validator = new ParametrizacaoModelValidator();
        ErrosValidacaoFormulario.Clear();
        List<string> erros = new();

        var results = validator.Validate(Parametrizacao);
        if (!results.IsValid)
        {
            foreach (var erro in results.Errors)
            {
                erros.Add(erro.ErrorMessage);
            }
            ErrosValidacaoFormulario.AddRange(erros);
            return false;
        }
        return true;
    }

    public void AoFecharDlg(object e)
    {
        //throw new System.NotImplementedException();
    }

    public void LimparTela()
    {
        //throw new System.NotImplementedException();
    }

    public void GetTiposEmissaoNFe()
    {
        TiposEmissaoNFe = UIExtensions.EnumDataSource(TipoEmissaoNFeACBr.teNormal);
    }

    public void GetTiposAmbienteNFe()
    {
        TiposAmbienteNFe = UIExtensions.EnumDataSource(TipoAmbienteNFeACBr.taHomologacao);
    }

    public async Task GetParametrizacao()
    {
        var resultConsulta = await _parametrizacoesService.GetAsync(URIsParametrizacoes.GetAsync);

        if (!resultConsulta.Sucesso)
        {
            await ExibirMensagemToast(resultConsulta.MensagemErro, TipoToast.Erro);
            return;
        }

        if (resultConsulta.Status == System.Net.HttpStatusCode.NoContent)
        {
            return;
        }

        Parametrizacao = (ParametrizacaoModel)resultConsulta.Objeto;
        string chave = _configuration.GetSection("EncryptKey").Value;
        Parametrizacao.SenhaCertificadoNFe = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.DecriptaString(chave, Parametrizacao.SenhaCertificadoNFe) : null;
        Parametrizacao.SenhaEMail = !string.IsNullOrEmpty(Parametrizacao.SenhaEMail) ? EncriptacaoDecriptacao.DecriptaString(chave, Parametrizacao.SenhaEMail) : null;
        AoMudarPropriedade(nameof(Parametrizacao));
    }

    public async Task AoSelecionarCertificadoUpload(UploadChangeEventArgs args)
    {
        await SpinnerPrincipal.ShowAsync();
        var resultUpload = await _uploadCertificadoDigitalService.Upload(args);
        await SpinnerPrincipal.HideAsync();

        if (!resultUpload.Sucesso)
        {
            await ExibirMensagemToast((string)resultUpload.Retorno, TipoToast.Erro);
            return;
        }

        string nmArq = (string)resultUpload.Retorno;
        Parametrizacao.CertificadoNFe = Path.GetFileName(nmArq);

        await ExibirMensagemToast("Upload realizado com sucesso.", TipoToast.Sucesso);
    }

    public async Task AoSelecionarDANFEUpload(UploadChangeEventArgs args)
    {
        await SpinnerPrincipal.ShowAsync();
        var resultUpload = await _uploadCertificadoDigitalService.Upload(args);
        await SpinnerPrincipal.HideAsync();

        if (!resultUpload.Sucesso)
        {
            await ExibirMensagemToast((string)resultUpload.Retorno, TipoToast.Erro);
            return;
        }

        string nmArq = (string)resultUpload.Retorno;
        Parametrizacao.CertificadoNFe = Path.GetFileName(nmArq);

        await ExibirMensagemToast("Upload realizado com sucesso.", TipoToast.Sucesso);
    }

    public async Task AoSelecionarLogoDANFENFeUpload(UploadChangeEventArgs args)
    {
        await SpinnerPrincipal.ShowAsync();
        var resultUpload = await _uploadLogoDANFENFeService.Upload(args);
        await SpinnerPrincipal.HideAsync();

        if (!resultUpload.Sucesso)
        {
            await ExibirMensagemToast((string)resultUpload.Retorno, TipoToast.Erro);
            return;
        }

        string nmArq = (string)resultUpload.Retorno;
        Parametrizacao.LogoDANFENFe = Path.GetFileName(nmArq);

        await ExibirMensagemToast("Upload realizado com sucesso.", TipoToast.Sucesso);
    }
}
